@@ -1,5 +1,6 @@
 import threading
 from typing import Any, Coroutine
+from arkitekt.actors.exceptions import ThreadedActorCancelled
 from arkitekt.messages.postman.assign.assign_log import AssignLogMessage
 import contextvars
 from arkitekt.messages.postman.assign.assign_critical import AssignCriticalMessage
@@ -8,7 +9,7 @@ from arkitekt.messages.postman.assign.assign_return import AssignReturnMessage
 from arkitekt.messages.postman.assign.assign_yield import AssignYieldsMessage
 from arkitekt.messages.postman.assign.assign_done import AssignDoneMessage
 from arkitekt.messages.postman.provide.bounced_provide import BouncedProvideMessage
-from arkitekt.threadvars import assign_message, transport, janus_queue
+from arkitekt.threadvars import assign_message, transport, janus_queue, cancel_event
 from arkitekt.messages.postman.assign.bounced_forwarded_assign import BouncedForwardedAssignMessage
 from arkitekt.actors.base import Actor
 from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
@@ -143,7 +144,7 @@ class FunctionalGenActor(FunctionalActor):
 
 
         except Exception as e:
-            self.console.print_exception()
+            logger.exception(e)
             await self.transport.forward(AssignCriticalMessage(data={
                 "type": e.__class__.__name__,
                 "message": str(e)
@@ -161,43 +162,56 @@ class FunctionalThreadedFuncActor(FunctionalActor):
         self.threadpool = ThreadPoolExecutor(nworkers)
 
     async def iterate_queue(self, async_q, message: BouncedForwardedAssignMessage):
-        while True:
-            val = await async_q.get()
-            action = val[0]
-            value = val[1]
+        try:
+            while True:
+                val = await async_q.get()
+                action = val[0]
+                value = val[1]
 
-            if action == "log":
-                message = AssignLogMessage(
-                    data = {
-                        "message": value[0],
-                        "level": value[1]
-                    },
-                    meta = {**message.meta.dict(exclude={"type"})}
-                )
+                if action == "log":
+                    message = AssignLogMessage(
+                        data = {
+                            "message": value[0],
+                            "level": value[1]
+                        },
+                        meta = {**message.meta.dict(exclude={"type"})}
+                    )
 
-                await self.transport.forward(message)
-                async_q.task_done()
-            if action == "return":
-                await self.transport.forward(AssignReturnMessage(data={
-                    "returns": await shrink_outputs(self.template.node, value) if self.shrink_outputs else value
-                }, meta = {**message.meta.dict(exclude={"type"})}
-                ))
-                async_q.task_done()
-                break
-            if action == "exception":
-                await self.transport.forward(AssignCriticalMessage(data={
-                        "type": val.__class__.__name__,
-                        "message": str(val)
+                    await self.transport.forward(message)
+                    async_q.task_done()
+                if action == "return":
+                    await self.transport.forward(AssignReturnMessage(data={
+                        "returns": await shrink_outputs(self.template.node, value) if self.shrink_outputs else value
                     }, meta = {**message.meta.dict(exclude={"type"})}
                     ))
-                async_q.task_done()
-                break  
+                    async_q.task_done()
+                    break
+                if action == "exception":
+                    async_q.task_done()
+                    raise value 
+
+            
+        except asyncio.CancelledError as cancelled_error:
+            print("Cancelled Here")
+            while True:
+                val = await async_q.get()
+                action = val[0]
+                value = val[1]
+                if action == "exception":
+                    async_q.task_done()
+                    print("Received Here")
+                    try:
+                        raise value
+                    except ThreadedActorCancelled:
+                        print("Caught here")
+                        raise cancelled_error
 
 
 
-    def _assign_threaded(self, queue, message, args, kwargs):
+    def _assign_threaded(self, queue, cancel_event_instance, message, args, kwargs):
         janus_queue.set(queue)
         assign_message.set(message)
+        cancel_event.set(cancel_event_instance)
         try:
             result = self.assign(*args, **kwargs)
             queue.put(("return", result))
@@ -210,33 +224,38 @@ class FunctionalThreadedFuncActor(FunctionalActor):
 
         janus_queue.set(None)
         assign_message.set(None)
+        cancel_event.set(None)
 
 
     async def on_assign(self, message: BouncedForwardedAssignMessage):
         queue = janus.Queue()
+        event = threading.Event()
 
         try:
             logger.info("Assigning Number two")
             args, kwargs = await expand_inputs(self.template.node, message.data.args, message.data.kwargs) if self.expand_inputs else (message.data.args, message.data.kwargs)
             
 
-            threadedfut = self.loop.run_in_executor(self.threadpool, self._assign_threaded,  queue.sync_q, message, args, kwargs)
+            threadedfut = self.loop.run_in_executor(self.threadpool, self._assign_threaded,  queue.sync_q,event,  message, args, kwargs)
             queuefut =  self.iterate_queue(queue.async_q, message)
 
             try:
-                await asyncio.gather(threadedfut, queuefut)
+                await asyncio.gather(asyncio.shield(threadedfut), asyncio.shield(queuefut))
                 queue.close()
                 await queue.wait_closed()
 
             except asyncio.CancelledError as e:
                 await self.log("Received Cancellation for task")
 
-                if not threadedfut.done():
-                    await self.log("Sending request to Queue To Cancell")
-                    threadedfut.cancel()
+                print("Nananananana")
+                queuefut.cancel() # We cancel the quefuture and are now only waiting for cancellation requests
+                event.set() # We are sending the request to the queue
 
+                print("CACACACACACA")
+                print("LALALALALALA")
                 try:
-                    await threadedfut
+                    await self.log("asdasdasd")
+                    await queuefut
                 except asyncio.CancelledError as e:
                     await self.log("Sucessfully Cancelled Thread")
                     raise e
@@ -274,72 +293,80 @@ class FunctionalThreadedGenActor(FunctionalActor):
 
     
     async def iterate_queue(self, async_q, message: BouncedForwardedAssignMessage):
-        while True:
-            val = await async_q.get()
-            action = val[0]
-            value = val[1]
+        try:
+            while True:
+                val = await async_q.get()
+                action = val[0]
+                value = val[1]
 
-            if action == "log":
-                message = AssignLogMessage(
-                    data = {
-                        "message": value[0],
-                        "level": value[1]
-                    },
-                    meta = {**message.meta.dict(exclude={"type"})}
-                )
+                if action == "log":
+                    message = AssignLogMessage(
+                        data = {
+                            "message": value[0],
+                            "level": value[1]
+                        },
+                        meta = {**message.meta.dict(exclude={"type"})}
+                    )
 
-                await self.transport.forward(message)
-                async_q.task_done()
-            if action == "yield":
-                await self.transport.forward(AssignYieldsMessage(data={
-                    "returns": await shrink_outputs(self.template.node, value) if self.shrink_outputs else value
-                }, meta = {**message.meta.dict(exclude={"type"})}
-                ))
-                async_q.task_done()
-            if action == "done":
-                await self.transport.forward(AssignDoneMessage(data={
-                }, meta = {**message.meta.dict(exclude={"type"})}
-                ))
-                async_q.task_done()
-            if action == "exception":
-                await self.transport.forward(AssignCriticalMessage(data={
-                        "type": val.__class__.__name__,
-                        "message": str(val)
+                    await self.transport.forward(message)
+                    async_q.task_done()
+                if action == "yield":
+                    await self.transport.forward(AssignYieldsMessage(data={
+                        "returns": await shrink_outputs(self.template.node, value) if self.shrink_outputs else value
                     }, meta = {**message.meta.dict(exclude={"type"})}
                     ))
-                async_q.task_done()
-                break  
+                    async_q.task_done()
+                if action == "done":
+                    await self.transport.forward(AssignDoneMessage(data={
+                    }, meta = {**message.meta.dict(exclude={"type"})}
+                    ))
+                    async_q.task_done()
+                    break
+                if action == "exception":
+                    async_q.task_done()
+                    print("Received Here")
+                    raise value
+
+        except asyncio.CancelledError as cancelled_error:
+            print("Cancelled Here")
+            while True:
+                val = await async_q.get()
+                action = val[0]
+                value = val[1]
+                if action == "exception":
+                    async_q.task_done()
+                    print("Received Here")
+                    try:
+                        raise value
+                    except ThreadedActorCancelled:
+                        print("Caught here")
+                        raise cancelled_error
+            
+                   
 
 
 
-    def _assign_threaded(self, queue, cancel_event, message, args, kwargs):
-        janus_queue.set(queue)
+    def _assign_threaded(self, sync_queue, cancel_event_instance, message, args, kwargs):
+        janus_queue.set(sync_queue)
         assign_message.set(message)
+        cancel_event.set(cancel_event_instance)
         try:
-            if cancel_event.is_set(): 
-                queue.put(("cancelled", "Cancelled before iteration"))
-                queue.join()
-                return
-
-            iteration = 0
             for result in self.assign(*args, **kwargs):
-                if cancel_event.is_set():
-                    queue.put(("cancelled",f"Cancelled after {iteration} Iterations"))
-                    queue.join()
-                    return
+                sync_queue.put(("yield", result))
+                sync_queue.join()
 
-                queue.put(("yield", result))
-                queue.join()
-
-            queue.put(("done","Happy doneness"))
+            sync_queue.put(("done","Happy doneness"))
+            sync_queue.join()
 
         except Exception as e:
+            print("Reeeeeeeellly???")
             logger.exception(e)
-            queue.put(("exception", e))
+            sync_queue.put(("exception", e))
+            sync_queue.join()
 
         janus_queue.set(None)
         assign_message.set(None) 
-
+        cancel_event.set(None)
 
 
     async def on_assign(self, message: BouncedForwardedAssignMessage):
@@ -352,24 +379,31 @@ class FunctionalThreadedGenActor(FunctionalActor):
             
 
             threadedfut = self.loop.run_in_executor(self.threadpool, self._assign_threaded,  queue.sync_q, event, message, args, kwargs)
-            queuefut =  self.iterate_queue(queue.async_q, message)
+            queuefut =  self.loop.create_task(self.iterate_queue(queue.async_q, message))
 
             try:
-                await asyncio.gather(threadedfut, queuefut)
+                await asyncio.gather(asyncio.shield(threadedfut), asyncio.shield(queuefut))
                 queue.close()
                 await queue.wait_closed()
 
             except asyncio.CancelledError as e:
                 await self.log("Received Cancellation for task")
 
-                if not threadedfut.done():
-                    await self.log("Sending request to Queue To Cancell")
-                    threadedfut.cancel()
+                print("Nananananana")
+                queuefut.cancel() # We cancel the quefuture and are now only waiting for cancellation requests
+                event.set() # We are sending the request to the queue
 
+                print("CACACACACACA")
+                print("LALALALALALA")
                 try:
-                    await threadedfut
+                    await self.log("asdasdasd")
+                    await queuefut
                 except asyncio.CancelledError as e:
                     await self.log("Sucessfully Cancelled Thread")
+
+                    queue.close()
+                    await queue.wait_closed()
+
                     raise e
 
         except asyncio.CancelledError as e:
