@@ -52,7 +52,7 @@ def main(script = ArkitektOptions.INIT, name=None, refresh=False):
                 fakts.load()
 
 
-        with open(os.path.join(directory, "run.py")) as f:
+        with open(os.path.join(directory, "run.py"), "w") as f:
             f.write(init_script)
 
 
