@@ -21,7 +21,7 @@ class PortsWrapped(QtWidgets.QWidget):
         qlayout = QtWidgets.QFormLayout()
 
         for port in ports:
-            argLabel = QtWidgets.QLabel(port.typename + " | " + port.key)
+            argLabel = QtWidgets.QLabel(f"{port.typename} | {port.key}")
             argDescription = QtWidgets.QLabel(port.description)
             argDescription.setFont(QtGui.QFont("Arial", 8))
             qlayout.addWidget(argLabel)
